@@ -1,7 +1,10 @@
-#!/usr/bin/env python
-# encoding: utf8
+#!/usr/bin/env python2
+# encoding: utf-8
 
 """
+NOTE: You should use wmlparser2.py instead which uses the C++
+preprocessor.
+
 This module represents the internal appearance of WML.
 
 Note: We cannot store textdomain information, due to how the parser work.
@@ -12,6 +15,7 @@ textdomain stuff in here is therefore only useful to CampGen, as that does
 not allow composed strings like above.
 """
 
+from __future__ import print_function
 import re, sys
 import wmlparser
 import codecs
@@ -35,7 +39,9 @@ class Data:
         pos = indent * "  "
         result = pos + "\ " + magenta + self.name + off + " (" + self.__class__.__name__ + ")"
         if show_contents:
-            result += "'" + self.get_value() + "'"
+            if self.is_translatable() == True:
+                result += " _"
+            result += " '" + self.get_value() + "'"
         if write:
             # The input usually is utf8, but it also may be not - for example
             # if a .png image is transmitted over WML. As this is only for
@@ -58,6 +64,9 @@ class Data:
     def get_value(self):
         return ""
 
+    def is_translatable(self):
+        return False
+
     def get_type(self):
         return self.__class__.__name__
 
@@ -84,6 +93,9 @@ class DataText(Data):
     def set_value(self, data):
         self.data = data
 
+    def is_translatable(self):
+        return self.translatable
+
 class DataBinary(Data):
     """A binary chunk of WML."""
     def __init__(self, name, binary):
@@ -146,7 +158,7 @@ class WMLException(Exception):
     def __init__(self, text):
         super( WMLException, self ).__init__()
         self.text = text
-        print text
+        print(text)
     def __str__(self):
         return self.text
 
@@ -171,12 +183,12 @@ class DataSub(Data):
                 item.clean_empty_ifdefs()
         while rem:
             item = rem.pop()
-            print "Removing empty #ifdef %s" % item.name
+            print("Removing empty #ifdef %s" % item.name)
             self.remove(item)
 
     def write_file(self, f, indent=0, textdomain=""):
         f.write(self.make_string( indent, textdomain))
-        
+
     def make_string(self, indent = 0, textdomain = ""):
         """Write the data object to the given file object."""
         ifdef = 0
@@ -264,7 +276,8 @@ class DataSub(Data):
         bytes = ""
         for r in result:
             if r != None:
-                # For networking, we need actual bytesteam here, not unicode.
+                # For networking, we need actual bytestream here, not unicode.
+                if type(r) is unicode: r = r.encode("utf8")
                 bytes += str(r)
 
         return bytes
@@ -356,7 +369,7 @@ class DataSub(Data):
 
     def compare(self, other):
         if len(self.data) != len(other.data): return False
-        for i in range(self.data):
+        for i in xrange(self.data):
             if not self.data[i].compare(other.data[i]): return False
         return True
 
@@ -469,6 +482,16 @@ class DataSub(Data):
             if isinstance(data, DataText): return data
         return None
 
+    def get_texts(self, name):
+        """Gets all text elements matching the name"""
+        return [text for text in self.get_all(name)
+            if isinstance(text, DataText)]
+
+    def get_all_text(self):
+        """Gets all text elements"""
+        return [text for text in self.data
+            if isinstance(text, DataText)]
+
     def get_binary(self, name):
         """Return a binary element"""
         for data in self.get_all(name):
@@ -483,7 +506,7 @@ class DataSub(Data):
         """Gets all macros matching the name"""
         return [macro for macro in self.get_all(name)
             if isinstance(macro, DataMacro)]
-            
+
     def get_all_macros(self):
         """Gets all macros"""
         return [macro for macro in self.data
@@ -495,10 +518,15 @@ class DataSub(Data):
             if isinstance(ifdef, DataIfDef)]
 
     def get_subs(self, name):
-        """Gets all macros matching the name"""
+        """Gets all elements matching the name"""
         return [sub for sub in self.get_all(name)
             if isinstance(sub, DataSub)]
 
+    def get_all_subs(self):
+        """Gets all elements"""
+        return [sub for sub in self.data
+            if isinstance(sub, DataSub)]
+
     def remove_macros(self, name):
         for macro in self.get_macros(name):
             self.remove(macro)
@@ -564,7 +592,7 @@ class DataSub(Data):
         value = str(value)
         # read existing values
         q = []
-        for d in range(3):
+        for d in xrange(3):
             q += [self.get_quantity(name, d, value)]
         q[difficulty] = value
 
@@ -577,7 +605,7 @@ class DataSub(Data):
         if q[0] == q[1] == q[2]:
             self.set_text_val(name, value)
         else:
-            for d in range(3):
+            for d in xrange(3):
                 ifdef = self.get_or_create_ifdef(["EASY", "NORMAL", "HARD"][d])
                 ifdef.set_text_val(name, q[d])
 
